// simulate/show how two tasks with similar periods drift
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h> // required for usleep

// one column is one character and represents one millisecond
#define NOF_COLS        (120)    // number of columns
#define NOF_ROWS        (2000)   // number of rows

#define PERIOD_1        (20.0f)  // period of calc in ms
#define PERIOD_2        (20.01f) // period of RX in ms
#define INITIAL_OFFSET  (8.7f)   // related to RX
#define JITTER_FACTOR   (2.5f)   // related to RX

int main(void)
{
  char buf[NOF_ROWS * NOF_COLS];
  int i;
  double pos, jitter, t = 0;

  memset(buf, ' ', sizeof(char) * NOF_COLS * NOF_ROWS);

  while (t < NOF_COLS * NOF_ROWS) { // firstly calc
    buf[(int)t] = '|'; // a "|" represents calc
    t += PERIOD_1;
  }

  pos = t = INITIAL_OFFSET;
  while (pos < NOF_COLS * NOF_ROWS) { // now RX
    if (buf[(int)pos] == ' ') {
      buf[(int)pos] = '-'; // a "-" represents RX
    } else {
      buf[(int)pos] = '+'; // "+" indicates calc and RX
    }

    jitter = JITTER_FACTOR * ((double)rand()/
         (double)RAND_MAX - 0.5f);

    t += PERIOD_2;
    pos = t + jitter; // only RX jitters
  }

  for (i=0; i<NOF_ROWS; i++) { // now print to stdout
    fwrite(&buf[i*NOF_COLS], sizeof(char), NOF_COLS, stdout);
    usleep(1000); // slow down output a bit
    printf("\n");
  }

  char last = ' ';
  for (i=0; i<NOF_ROWS * NOF_COLS; i++) {
    if (buf[i] == '-') {
      if (last == '-') {
        printf("double - at %dms\n", i);
      }
      last = '-';
    }

    if (buf[i] == '|') {
      if (last == '|') {
        printf("double | at %dms\n", i);
      }
      last = '|';
    }
  }

  return 0;
}
